#include <iostream>
#include <iomanip>
#include <string>
#include <sstream>
#include <fstream>

#include <vector>
#include <stdint.h>
#include <stdio.h>
#include <math.h>

using namespace std;

// Used to find the start of each data group
const char data_sentinel = '#';
const char gps_sentinel = '$';
const int data_num=7;

int main(int argc, char **argv)
{
  // UTC from GPS string
  float world_time=0;
  float alt_world_time=0;
  float prev_world_time=0; 
  float output_time;

  int temp1,temp2,temp3,temp4,temp5;

  int count=1;

  // To hold the data
  float delta_time;
  vector<vector<int16_t> > temp;
  vector<double> temp_timestamps;

  uint32_t time; // 4 bytes (unsigned int)
  uint32_t gps_time; // 4 bytes (unsigned int)
  vector<int16_t> data; // 2 bytes each (short)
  int16_t data_array[data_num];
  char c = '\0'; // 1 byte
  
  fstream gps_file; // Output file for parsed GPS data
  fstream cmp_file; // Output file for the parsed compass data
  
  gps_file.open("./gps_file.txt",fstream::out);
  cmp_file.open("./data_file.txt",fstream::out);
    
  // Convert the data into text and output it to the data buffer
  do {
        // Find the start of a line
    while ((c != data_sentinel) && (c != gps_sentinel) && !cin.eof())
      cin.read((&c),sizeof(c));
    if (cin.eof()) return 0;
    
    // Parse a data line
    if (c == data_sentinel)
      {

	//read in micros() timestamp
	cin.read(reinterpret_cast<char *>(&time),sizeof(time));
	for (int i = 0; i < data_num; i++)
	  {
	    //read in acc/mag data
	    cin.read(reinterpret_cast<char *>(&data_array[i]),sizeof(data_array[i]));
	    data.push_back(data_array[i]);
	    if (cin.eof()){ 
	      return 0;
	    }
	  }
	
	if(world_time != 0){
	  
	  //store line of data in temporary data vector
	  temp.push_back(data);
	  data.clear();

	  double timestamp=static_cast<double>(time-gps_time)/1000000;
	  temp_timestamps.push_back(timestamp);

	  //check to see if the gps time --- world_time --- has been updated


	  if(alt_world_time!=world_time){

	    //if the difference between current time and previous time is arbitrarily large, don't spread the data points accross the whole gap because that will lead to too much error. Instead use the timestamp
	    int backup=1;
	    if(world_time-prev_world_time > 2){
	      delta_time=0;
	    }
	    else{
	      delta_time=(world_time-prev_world_time)/count;
	      backup=0;
	    }

	    //use the calculated delta_time and count, which represents the number of compass data points between a given pair of gps data points, to determine and print out the appropriate time for each compass data point 
	  
	    for(int i=0;i<count-1;i++){
	      streamsize ss = cout.precision();
	      cmp_file.precision(10);

	      output_time=prev_world_time+i*delta_time+backup*temp_timestamps[i];

	              if(output_time >= 52450 && output_time <= 58518 && temp[i][0] < 1000 && temp[i][1] < 5000 && temp[i][1] > -5000 && temp[i][2] < 5000 && temp[i][3] < 5000 && temp[i][4] < 10000 && temp[i][5] < 10000 && temp[i][6] < 10000){ 

	

		cmp_file << output_time << ' ' ;
	       
		cmp_file.precision(ss);
		for(int j=0;j<data_num;j++){
		  cmp_file << temp[i][j] << ' ';
		}
		cmp_file <<  endl;
		  }
	    }
	    
	    //reset count, alt_world_time, and temp
	    count=1;
	    alt_world_time=world_time;
	    temp.clear();
	    temp_timestamps.clear();
	  }
	  //update count
	  count++;
	}
	else{
	  data.clear();
	}
	if (cin.eof()) return 0;
       
	// Update c so it is no longer the previous sentinel
	cin.read((&c),sizeof(c)); 
      }
    else // Handle a GPS string
      {

	// micros() timestamp
	cin.read(reinterpret_cast<char *>(&gps_time),sizeof(gps_time));
	if (cin.eof()) return 0;
        
	// To temporarily store the GPS tring
	stringstream gps_buf;
            
	cin.read((&c),sizeof(c));
	while ((c != data_sentinel) && !cin.eof())
	  {
	    gps_buf << c;
	    cin.read((&c),sizeof(c));
	  }
	if (cin.eof()) return 0;
	
	/* Code to parse GPS data */
	// Values to read fromt the GPS string
	float timef;
	float lat; char a1[2];
	float lng; char a2[2];
	int stat; int nSat;
	float hor; float altf;
        
	//read in the gps data
	sscanf(gps_buf.str().c_str(),"GPGGA,%f,%f,%1s,%f,%1s,%d,%d,%f,%f", &timef, &lat, a1, &lng, a2, &stat, &nSat, &hor, &altf);
	
	//	cout << timef << " " << lat << endl;

	// Convert the time to hh:mm:ss format
	int timeg = round(timef);

	if(timeg>100){

	int hr = timeg/10000;
	int mi = timeg/100-hr*100;
	int sc = timeg-hr*10000-mi*100;

	//update world_time and prev_world_time. Serves as a more reliable time than the micros timestamp.
	//fill in alternative value if hr is not within the flight time.
	if(hr < 14 || hr > 16){
	  prev_world_time=world_time;
	  world_time = world_time+1;
	}
	else{
	  prev_world_time=world_time;
	  world_time = 3600*hr+60*mi+sc;
	}

	// Get GPS coordinates entirely in degrees
	int deg = round(lat/100);
	float min = lat - deg*100;
	lat = deg + min/60.0;
        
	deg = round(lng/100);
	min = lng - deg*100;
	lng = -1.0*(deg+min/60.0); // -1 really comes from a1 and a2, but we don't need to check because we're always in New England so it doesn't change
        
	int alt = round(altf);
	if (alt != 0)
	  {
	    gps_file << gps_time << " " << setfill('0') << setw(2) << hr << " " << setw(2) << mi << " " << setw(2) << sc << "\t" << lat << " " << lng << " " << alt << endl;
	  }
	}
	/* end GPS parsing code */
      }
  } while (cin && !cin.eof());
  
  gps_file.close();
  cmp_file.close();
}
